@@ -3,11 +3,13 @@
 module Main (main) where
 
 import           A
-import           Control.DeepSeq (NFData (..))
+import           Control.DeepSeq      (NFData (..))
 import           Criterion.Main
-import qualified Data.Text.IO    as TIO
+import qualified Data.ByteString.Lazy as BSL
+import qualified Data.Text.IO         as TIO
 import           File
-import           System.IO       (IOMode (WriteMode), withFile)
+import           Jacinda.Regex
+import           System.IO            (IOMode (WriteMode), withFile)
 
 hrun ifp e m fp = withFile "/dev/null" WriteMode $ \h -> runOnFile [] Nothing e [] m fp h
 runs e m fp = nfIO $ hrun "(bench)" e m fp
@@ -23,6 +25,11 @@ main =
                       [ bench "dedup" $ runs "~.{ix>1}{`8}" CSV "bench/data/food-prices.csv"
                       , bench "succdiff" $ runs "(%)\\. {%/Apple/}{`3:}" CSV "bench/data/food-prices.csv"
                       ]
+                , let rp=tcompile "\n\n"
+                  in bgroup "rure"
+                      [ bench "RS" $ nfIO (do {contents <- BSL.readFile "bench/data/ghc"; pure (lazySplit rp contents)})
+                      , bench "header" $ nfIO (do {contents <- BSL.readFile "bench/data/ghc"; pure (lazySplitH rp contents)})
+                      ]
                 , bgroup "report"
                       [ bench "ghc-filt" $ fruns "test/examples/ghc-filt.jac" awk "test/data/ghc" ]
                 , bgroup "stream"
