@@ -35,7 +35,7 @@ infixr 6 <##>
 data TB = TyI | TyFloat | TyStr
         | TyStream | TyVec | TyOption
         | TyR | TyBool | TyUnit
-        deriving (Eq, Ord)
+        deriving Eq
 
 tupledByFunky :: Doc ann -> [Doc ann] -> Doc ann
 tupledByFunky sep = group . encloseSep (flatAlt "⟨ " "⟨") (flatAlt " ⟩" "⟩") sep
@@ -55,14 +55,14 @@ infixr 0 ~>
 
 infixr 0 :$
 
-data T = TyB { tyBuiltin :: TB }
+data T = TyB { tyBuiltin :: !TB }
        | (:$) { tyApp0, tyApp1 :: T }
        | TyArr { tyArr0, tyArr1 :: T }
-       | TyVar { tyVar :: Nm () }
+       | TyVar { tyVar :: !(Nm ()) }
        | TyTup { tyTups :: [T] }
        | TyRec { tyres :: NmMap T }
-       | Rho { tyRho :: Nm (), tyArms :: IM.IntMap T }
-       | Ρ { tyΡ :: Nm (), tyρs :: NmMap T }
+       | Rho { tyRho :: !(Nm ()), tyArms :: IM.IntMap T }
+       | Ρ { tyΡ :: !(Nm ()), tyρs :: NmMap T }
        deriving Eq
 
 instance Pretty TB where
@@ -100,7 +100,6 @@ data BUn = Tally -- length of string field
          | Negate
          | TallyList -- length of vector
          | Head | Tail | Init | Last
-         deriving (Eq)
 
 instance Pretty BUn where
     pretty Tally      = "#"
@@ -129,8 +128,7 @@ data BTer = ZipW
           | Substr | Sub1 | Subs
           | Option
           | Captures | AllCaptures | Ixes
-          | Bookend
-          deriving (Eq)
+          | Bookend | BookendG
 
 instance Pretty BTer where
     pretty ZipW        = ","
@@ -162,7 +160,6 @@ data BBin = Plus | Times | Div
           | Report
           | Take | Drop
           | Rein | Nier
-          deriving (Eq)
 
 instance Pretty BBin where
     pretty Plus = "+"; pretty Times = "*"; pretty Div = "%"; pretty Minus = "-"
@@ -176,21 +173,37 @@ instance Pretty BBin where
     pretty Take = "take#"; pretty Drop = "drop#"; pretty Rein = "reintercalate"
     pretty Nier = "@@"
 
-data DfnVar = X | Y deriving (Eq)
+data DfnVar = X | Y
 
 instance Pretty DfnVar where pretty X="x"; pretty Y="y"
 
 -- 0-ary
-data N = Ix | Nf | None | Fp | MZ deriving (Eq)
+data N = Ix | Nf | None | Fp | MZ
 
-data L = ILit !Integer | FLit !Double | BLit !Bool | StrLit BS.ByteString deriving (Generic, NFData, Eq)
+data L = ILit !Integer | FLit !Double | BLit !Bool | StrLit BS.ByteString deriving (Generic, Eq, Ord, NFData)
 
 class PS a where ps :: Int -> a -> Doc ann
 
+instance Eq (E a) where
+    (==) (Lit _ l₀) (Lit _ l₁)             = l₀==l₁
+    (==) (Tup _ es₀) (Tup _ es₁)           = es₀==es₁
+    (==) (Rec _ a₀) (Rec _ a₁)             = a₀==a₁
+    (==) (OptionVal _ e₀) (OptionVal _ e₁) = e₀==e₁
+    (==) (Arr _ e₀) (Arr _ e₁)             = e₀==e₁
+    (==) _ _                               = undefined
+
+instance Ord (E a) where
+    compare (Lit _ l₀) (Lit _ l₁)             = compare l₀ l₁
+    compare (Tup _ es₀) (Tup _ es₁)           = compare es₀ es₁
+    compare (Rec _ a₀) (Rec _ a₁)             = compare a₀ a₁
+    compare (OptionVal _ e₀) (OptionVal _ e₁) = compare e₀ e₁
+    compare (Arr _ e₀) (Arr _ e₁)             = compare e₀ e₁
+    compare _ _                               = undefined
+
 -- expression
-data E a = Column { eLoc :: a, col :: Int }
-         | IParseCol { eLoc :: a, col :: Int } | FParseCol { eLoc :: a, col :: Int } | ParseCol { eLoc :: a, col :: Int }
-         | Field { eLoc :: a, eField :: Int } | LastField { eLoc :: a } | FieldList { eLoc :: a }
+data E a = Column { eLoc :: a, col :: !Int }
+         | IParseCol { eLoc :: a, col :: !Int } | FParseCol { eLoc :: a, col :: !Int } | ParseCol { eLoc :: a, col :: Int }
+         | Field { eLoc :: a, eField :: !Int } | LastField { eLoc :: a } | FieldList { eLoc :: a }
          | AllField { eLoc :: a } -- ^ Think @$0@ in awk.
          | AllColumn { eLoc :: a } -- ^ Think @$0@ in awk.
          | IParseAllCol { eLoc :: a } -- ^ @$0@, parsed as an integer
@@ -206,18 +219,18 @@ data E a = Column { eLoc :: a, col :: Int }
          | RegexLit { eLoc :: a, eRr :: BS.ByteString }
          | Lam { eLoc :: a, eBound :: Nm a, lamE :: E a }
          | Dfn { eLoc :: a, eDfn :: E a }
-         | BB { eLoc :: a, eBin :: BBin } | TB { eLoc :: a, eTer :: BTer } | UB { eLoc :: a, eUn :: BUn }
-         | NB { eLoc :: a, eNil :: N }
+         | BB { eLoc :: a, eBin :: !BBin } | TB { eLoc :: a, eTer :: !BTer } | UB { eLoc :: a, eUn :: !BUn }
+         | NB { eLoc :: a, eNil :: !N }
          | Tup { eLoc :: a, esTup :: [E a] }
          | Rec { eLoc :: a, esR :: [(Nm a, E a)] }
-         | ResVar { eLoc :: a, dfnVar :: DfnVar }
+         | ResVar { eLoc :: a, dfnVar :: !DfnVar }
          | RC RurePtr -- compiled regex after normalization
          | Arr { eLoc :: a, elems :: !(V.Vector (E a)) }
          | Anchor { eLoc :: a, eAnchored :: [E a] }
-         | Paren { eLoc :: a, eExpr :: E a }
+         | Paren { eLoc :: a, eExpr :: !(E a) }
          | OptionVal { eLoc :: a, eMaybe :: Maybe (E a) }
          | Cond { eLoc :: a, eIf, eThen, eElse :: E a }
-         | RwB { eLoc :: a, eBin :: BBin } | RwT { eLoc :: a, eTer :: BTer }
+         | RwB { eLoc :: a, eBin :: !BBin } | RwT { eLoc :: a, eTer :: !BTer }
          deriving (Functor)
 
 instance Pretty N where
@@ -278,6 +291,7 @@ instance PS (E a) where
     ps _ (EApp _ (UB _ FParse) e')    = pretty e' <> ":f"
     ps _ (EApp _ (UB _ Parse) e')     = pretty e' <> ":"
     ps d (EApp _ (EApp _ (EApp _ (TB _ Bookend) e) e') e'')  = parensp (d>3) (ps 4 e <> ",," <> ps 4 e' <+> ps 5 e'')
+    ps d (EApp _ (EApp _ (EApp _ (TB _ BookendG) e) e') e'') = parensp (d>3) (ps 4 e <> "|," <> ps 4 e' <+> ps 5 e'')
     ps d (EApp _ (EApp _ (EApp _ (TB _ Fold) e) e') e'')     = parensp (d>5) (ps 6 e <> "|" <> ps 6 e' <+> ps 7 e'')
     ps d (EApp _ (EApp _ (EApp _ (TB _ Scan) e) e') e'')     = parensp (d>5) (ps 6 e <> "^" <> ps 6 e' <+> ps 7 e'')
     ps d (EApp _ (EApp _ (EApp _ (TB _ ScanList) e) e') e'') = parensp (d>5) (ps 6 e <> "^" <> ps 6 e' <+> ps 7 e'')
@@ -318,41 +332,7 @@ instance Pretty (E a) where pretty=ps 0
 
 instance Show (E a) where show=show.pretty
 
--- for tests
-instance Eq (E a) where
-    (==) (Column _ i) (Column _ j)              = i == j
-    (==) (IParseCol _ i) (IParseCol _ j)        = i == j
-    (==) (FParseCol _ i) (FParseCol _ j)        = i == j
-    (==) (Field _ i) (Field _ j)                = i == j
-    (==) LastField{} LastField{}                = True
-    (==) FieldList{} FieldList{}                = True
-    (==) AllColumn{} AllColumn{}                = True
-    (==) AllField{} AllField{}                  = True
-    (==) (EApp _ e0 e1) (EApp _ e0' e1')        = e0 == e0' && e1 == e1'
-    (==) (Guarded _ p e) (Guarded _ p' e')      = p == p' && e == e'
-    (==) (Implicit _ e) (Implicit _ e')         = e == e'
-    (==) (Let _ (n, eϵ) e) (Let _ (n', eϵ') e') = eqName n n' && e == e' && eϵ == eϵ'
-    (==) (Var _ n) (Var _ n')                   = eqName n n'
-    (==) (Lam _ n e) (Lam _ n' e')              = eqName n n' && e == e'
-    (==) (Lit _ l) (Lit _ l')                   = l == l'
-    (==) (RegexLit _ rr) (RegexLit _ rr')       = rr == rr'
-    (==) (BB _ b) (BB _ b')                     = b == b'
-    (==) (TB _ b) (TB _ b')                     = b == b'
-    (==) (UB _ unOp) (UB _ unOp')               = unOp == unOp'
-    (==) (NB _ x) (NB _ y)                      = x == y
-    (==) (Tup _ es) (Tup _ es')                 = es == es'
-    (==) (Rec _ es) (Rec _ es')                 = es == es'
-    (==) (ResVar _ x) (ResVar _ y)              = x == y
-    (==) (Dfn _ f) (Dfn _ g)                    = f == g -- we're testing for lexical equivalence
-    (==) RC{} _                                 = error "Cannot compare compiled regex!"
-    (==) _ RC{}                                 = error "Cannot compare compiled regex!"
-    (==) (Paren _ e) e'                         = e == e'
-    (==) e (Paren _ e')                         = e == e'
-    (==) (RwB _ b) (RwB _ b')                   = b == b'
-    (==) (RwT _ b) (RwT _ b')                   = b == b'
-    (==) _ _                                    = False
-
-data C = IsNum | IsEq | IsOrd
+data C = IsNum | IsOrd
        | IsParse | IsPrintf
        | IsSemigroup | IsMonoid
        | Functor -- ^ For map (@"@)
@@ -360,7 +340,7 @@ data C = IsNum | IsEq | IsOrd
        deriving (Eq, Ord)
 
 instance Pretty C where
-    pretty IsNum = "Num"; pretty IsEq = "Eq"; pretty IsOrd = "Ord"
+    pretty IsNum = "Num"; pretty IsOrd = "Ord"
     pretty IsParse = "Parseable"; pretty IsSemigroup = "Semigroup"
     pretty Functor = "Functor"; pretty Foldable = "Foldable"
     pretty IsPrintf = "Printf"; pretty Witherable = "Witherable"
@@ -369,11 +349,11 @@ instance Pretty C where
 instance Show C where show=show.pretty
 
 -- decl
-data D a = SetFS T.Text | SetRS T.Text
+data D a = SetFS !T.Text | SetRS !T.Text
          | FunDecl (Nm a) [Nm a] (E a)
          | FlushDecl | SetH
          | SetAsv | SetUsv | SetCsv
-         | SetOFS T.Text | SetORS T.Text
+         | SetOFS !T.Text | SetORS !T.Text
          deriving (Functor)
 
 instance Pretty (D a) where
@@ -400,7 +380,7 @@ flushD (Program ds _) = any p ds where p FlushDecl = True; p _ = False
 
 awk = AWK Nothing Nothing False
 
-data Mode = CSV | AWK (Maybe T.Text) (Maybe T.Text) Bool -- field, record, include header in record split
+data Mode = CSV | AWK !(Maybe T.Text) !(Maybe T.Text) !Bool -- field, record, include header in record split
 
 getS :: Program a -> Mode
 getS (Program ds _) = foldl' go awk ds where
