@@ -55,13 +55,13 @@ data StreamError = NakedField deriving (Show)
 instance Exception StreamError where
 
 type Env = IM.IntMap (Maybe (E T)); type I=Int
-data Σ = Σ !I !Env (IM.IntMap (S.Set BS.ByteString)) (IM.IntMap IS.IntSet) (IM.IntMap (S.Set Double)) IS.IntSet
+data Σ = Σ !I !Env (IM.IntMap (S.Set BS.ByteString)) (IM.IntMap IS.IntSet) (IM.IntMap (S.Set Double)) IS.IntSet (IM.IntMap (S.Set (E T)))
 type Tmp = Int
 type Β = IM.IntMap (E T)
 
 mE :: (Env -> Env) -> Σ -> Σ
-mE f (Σ i e d di df b) = Σ i (f e) d di df b
-gE (Σ _ e _ _ _ _) = e
+mE f (Σ i e d di df b de) = Σ i (f e) d di df b de
+gE (Σ _ e _ _ _ _ _) = e
 
 at :: V.Vector a -> Int -> a
 v `at` ix = case v V.!? (ix-1) of {Just x -> x; Nothing -> throw $ IndexOutOfBounds ix}
@@ -102,19 +102,19 @@ run _ _ _ e _ | ty@TyArr{} <- eLoc e = error ("Found function type: " ++ show ty
 run h flush j e ctxs | TyB TyUnit <- eLoc e =  (\(s, f, env) -> pSF h flush (s,f) env) $ uStream j $ do
     (res, tt, iEnv, μ) <- unit e
     u <- nI
-    let outs=μ<$>ctxs; es'=scanl' (&) (Σ u iEnv IM.empty IM.empty IM.empty IS.empty) outs
+    let outs=μ<$>ctxs; es'=scanl' (&) (Σ u iEnv IM.empty IM.empty IM.empty IS.empty IM.empty) outs
     pure (res, tt, gE<$>es')
 run h flush j e ctxs | TyB TyStream:$_ <- eLoc e = traverse_ (traverse_ (pS h flush)).uStream j $ do
     t <- nI
     (iEnv, μ) <- ctx e t
     u <- nI
-    let outs=μ<$>ctxs; es={-# SCC "scanMain" #-} scanl' (&) (Σ u iEnv IM.empty IM.empty IM.empty IS.empty) outs
+    let outs=μ<$>ctxs; es={-# SCC "scanMain" #-} scanl' (&) (Σ u iEnv IM.empty IM.empty IM.empty IS.empty IM.empty) outs
     pure ((! t).gE<$>es)
 run h _ j e ctxs = pDocLn h $ uStream j $ do
     (iEnv, g, e0) <- collect e
     u <- nI
     let updates=g<$>ctxs
-        finEnv=foldl' (&) (Σ u iEnv IM.empty IM.empty IM.empty IS.empty) updates
+        finEnv=foldl' (&) (Σ u iEnv IM.empty IM.empty IM.empty IS.empty IM.empty) updates
     e0@>(fromMaybe (throw EmptyFold)<$>gE finEnv)
 
 unit :: E T -> UM (Maybe Tmp, [Tmp], Env, LineCtx -> Σ -> Σ)
@@ -276,6 +276,7 @@ ctx (EApp _ (EApp _ (EApp _ (TB _ Scan) op) seed) xs) o  = do {t <- nI; (env, sb
 ctx (EApp _ (EApp _ (EApp _ (TB _ ZipW) op) xs) ys) o    = do {t0 <- nI; t1 <- nI; (env0, sb0) <- ctx xs t0; (env1, sb1) <- ctx ys t1; pure (na o (env0<>env1), \l->wZ op t0 t1 o.sb0 l.sb1 l)}
 ctx (EApp _ (EApp _ (BB _ Prior) op) xs) o               = do {t <- nI; (env, sb) <- ctx xs t; pt <- nI; pure (na o (pt\~env), \l -> wΠ op pt t o.sb l)}
 ctx (EApp (_:$TyB ty) (UB _ Dedup) xs) o                 = do {k <- nI; t <- nI; (env, sb) <- ctx xs t; pure (na o env, \l->wD ty k t o.sb l)}
+ctx (EApp _ (UB _ Dedup) xs) o                           = do {k <- nI; t <- nI; (env, sb) <- ctx xs t; pure (na o env, \l->wDE k t o.sb l)}
 ctx (EApp _ (EApp _ (BB _ DedupOn) f) xs) o              = do {k <- nI; t <- nI; (env, sb) <- ctx xs t; pure (na o env, \l->wDOp f k t o.sb l)}
 ctx (EApp _ (EApp _ (EApp _ (TB _ Bookend) e0) e1) xs) o = do {k <- nI; t <- nI; (env, sb) <- ctx xs t; r0 <- e0@>mempty; r1<- e1@>mempty; pure (na o env, \l->wB (r0,r1) k t o.sb l)}
 ctx e _ | TyB TyStream:$_ <- eLoc e = error ("?? uh-oh. " ++ show e)
@@ -365,24 +366,12 @@ e@Lit{} @> _     = pure e
 e@RC{} @> _      = pure e
 (F n) @> b       = pure $ b!>n
 e@(Var _ n) @> b = pure $ case IM.lookup (unU$unique n) b of {Just y -> y; Nothing -> e}
-(EApp _ (EApp _ (BB (TyArr (TyB TyI) _) Max) x0) x1) @> b = do
-    x0' <- asI<$>(x0@>b); x1' <- asI<$>(x1@>b)
-    pure $ mkI (max x0' x1')
-(EApp _ (EApp _ (BB (TyArr (TyB TyI) _) Min) x0) x1) @> b = do
-    x0' <- asI <$> (x0@>b); x1' <- asI <$> (x1@>b)
-    pure $ mkI (min x0' x1')
-(EApp _ (EApp _ (BB (TyArr (TyB TyFloat) _) Max) x0) x1) @> b = do
-    x0' <- asF<$>(x0@>b); x1' <- asF<$>(x1@>b)
-    pure $ mkF (max x0' x1')
-(EApp _ (EApp _ (BB (TyArr (TyB TyFloat) _) Min) x0) x1) @> b = do
-    x0' <- asF<$>(x0@>b); x1' <- asF<$>(x1@>b)
-    pure $ mkF (min x0' x1')
-(EApp _ (EApp _ (BB (TyArr (TyB TyStr) _) Max) x0) x1) @> b = do
-    x0' <- asS<$>(x0@>b); x1' <- asS<$>(x1@>b)
-    pure $ mkStr (max x0' x1')
-(EApp _ (EApp _ (BB (TyArr (TyB TyStr) _) Min) x0) x1) @> b = do
-    x0' <- asS<$>(x0@>b); x1'<-asS<$>(x1@>b)
-    pure $ mkStr (min x0' x1')
+(EApp _ (EApp _ (BB _ Max) x0) x1) @> b = do
+    x0' <- x0@>b; x1' <- x1@>b
+    pure $ max x0' x1'
+(EApp _ (EApp _ (BB _ Min) x0) x1) @> b = do
+    x0' <- x0@>b; x1' <- x1@>b
+    pure $ min x0' x1'
 (EApp _ (EApp _ (BB (TyArr (TyB TyI) _) op) x0) x1) @> b | Just op' <- num op = do
     x0e <- asI<$>(x0@>b); x1e <- asI<$>(x1@>b)
     pure $ mkI (op' x0e x1e)
@@ -392,21 +381,9 @@ e@(Var _ n) @> b = pure $ case IM.lookup (unU$unique n) b of {Just y -> y; Nothi
 (EApp _ (EApp _ (BB _ Div) x0) x1) @> b = do
     x0e <- x0@>b; x1e <- x1@>b
     pure (mkF (asF x0e/asF x1e))
-(EApp _ (EApp _ (BB (TyArr (TyB TyI) _) op) x0) x1) @> b | Just rel <- binRel op = do
-    x0e<-asI<$>(x0@>b); x1e<-asI<$>(x1@>b)
-    pure (mkB (rel x0e x1e))
-(EApp _ (EApp _ (BB (TyArr (TyB TyFloat) _) op) x0) x1) @> b | Just rel <- binRel op = do
-    x0e <- asF<$>(x0@>b); x1e <- asF<$>(x1@>b)
-    pure (mkB (rel x0e x1e))
-(EApp _ (EApp _ (BB (TyArr (TyB TyStr) _) op) x0) x1) @> b | Just rel <- binRel op = do
-    x0e <- asS<$>(x0@>b); x1e <- asS<$>(x1@>b)
-    pure (mkB (rel x0e x1e))
-(EApp _ (EApp _ (BB (TyArr (TyB TyOption:$t@(TyB TyStr)) _) Eq) x0) x1) @> b = do
-    x0e <- asM<$>(x0@>b); x1e <- asM<$>(x1@>b)
-    case (x0e,x1e) of
-        (Nothing, Nothing)   -> pure (mkB True)
-        (Just e0b, Just e1b) -> (EApp tyB (EApp (t~>tyB) (BB (TyArr t (t~>t~>tyB)) Eq) e0b) e1b) @> b
-        _                    -> pure (mkB False)
+(EApp _ (EApp _ (BB _ op) x0) x1) @> b | Just rel <- binRel op = do
+    x0' <- x0@>b; x1' <- x1@>b
+    pure (mkB (rel x0' x1'))
 (EApp _ (EApp _ (BB (TyArr (TyB TyStr) _) Plus) x0) x1) @> b = do
     x0e <- x0@>b; x1e <- x1@>b
     pure (mkStr (asS x0e<>asS x1e))
@@ -558,70 +535,71 @@ ms :: Nm T -> E T -> Β
 ms (Nm _ (U i) _) = IM.singleton i
 
 wCM :: Tmp -> Tmp -> Σ -> Σ
-wCM src tgt (Σ u env d di df b) =
+wCM src tgt (Σ u env d di df b de) =
     Σ u (case env!src of
         Just y  -> case asM y of {Nothing -> tgt\~env; Just yϵ -> env&tgt~!yϵ}
-        Nothing -> tgt\~env) d di df b
+        Nothing -> tgt\~env) d di df b de
 
 {-# SCC wMM #-}
 wMM :: E T -> Tmp -> Tmp -> Σ -> Σ
-wMM (Lam _ n e) src tgt (Σ j env d di df b) =
+wMM (Lam _ n e) src tgt (Σ j env d di df b de) =
     case env!src of
         Just x ->
             let be=ms n x; (y,k)=e@!(j,be)
             in Σ k (case asM y of
                 Just yϵ -> env&tgt~!yϵ
-                Nothing -> tgt\~env) d di df b
-        Nothing -> Σ j (tgt\~env) d di df b
+                Nothing -> tgt\~env) d di df b de
+        Nothing -> Σ j (tgt\~env) d di df b de
 wMM e _ _ _ = throw$InternalArityOrEta 1 e
 
 wZ :: E T -> Tmp -> Tmp -> Tmp -> Σ -> Σ
-wZ (Lam _ n0 (Lam _ n1 e)) src0 src1 tgt (Σ j env d di df b) =
+wZ (Lam _ n0 (Lam _ n1 e)) src0 src1 tgt (Σ j env d di df b de) =
     (case (env!src0, env!src1) of
         (Just x, Just y) ->
             let be=me [(n0, x), (n1, y)]; (z,k)=e@!(j,be)
             in Σ k (env&tgt~!z)
-        (Nothing, Nothing) -> Σ j (tgt\~env)) d di df b
+        (Nothing, Nothing) -> Σ j (tgt\~env)) d di df b de
 wZ e _ _ _ _ = throw$InternalArityOrEta 2 e
 
 wM :: E T -> Tmp -> Tmp -> Σ -> Σ
-wM (Lam _ n e) src tgt (Σ j env d di df b) =
+wM (Lam _ n e) src tgt (Σ j env d di df b de) =
     case env!src of
         Just x ->
             let be=ms n x; (y,k)=e@!(j,be)
-            in Σ k (env&tgt~!y) d di df b
-        Nothing -> Σ j (tgt\~env) d di df b
+            in Σ k (env&tgt~!y) d di df b de
+        Nothing -> Σ j (tgt\~env) d di df b de
 wM e _ _ _ = throw$InternalArityOrEta 1 e
 
 wI :: E T -> Tmp -> LineCtx -> Σ -> Σ
-wI e tgt line (Σ j env d di df b) =
-    let e'=e `κ` line; (e'',k)=e'$@j in Σ k (env&tgt~!e'') d di df b
+wI e tgt line (Σ j env d di df b de) =
+    let e'=e `κ` line; (e'',k)=e'$@j in Σ k (env&tgt~!e'') d di df b de
 
 wG :: (E T, E T) -> Tmp -> LineCtx -> Σ -> Σ
-wG (p, e) tgt line (Σ j env d di df b) =
+wG (p, e) tgt line (Σ j env d di df b de) =
     let p'=p `κ` line; (p'',k)=p'$@j
     in (if asB p''
         then let e'=e `κ` line; (e'',u) =e'$@k in Σ u (env&tgt~!e'')
-        else Σ k (tgt\~env)) d di df b
+        else Σ k (tgt\~env)) d di df b de
 
+-- TODO: TyBool
 wDOp :: E T -> Int -> Tmp -> Tmp -> Σ -> Σ
-wDOp (Lam (TyArr _ (TyB TyStr)) n e) key src tgt (Σ i env d di df b) =
+wDOp (Lam (TyArr _ (TyB TyStr)) n e) key src tgt (Σ i env d di df b de) =
     case env!src of
-        Nothing -> Σ i (tgt\~env) d di df b
+        Nothing -> Σ i (tgt\~env) d di df b de
         Just xϵ ->
             case IM.lookup key d of
-                Nothing -> Σ k (env&tgt~!y) (IM.insert key (S.singleton e') d) di df b
-                Just ss -> (if e' `S.member` ss then Σ k (tgt\~env) d else Σ k (env&tgt~!y) (key!:e'$d)) di df b
+                Nothing -> Σ k (env&tgt~!y) (IM.insert key (S.singleton e') d) di df b de
+                Just ss -> (if e' `S.member` ss then Σ k (tgt\~env) d else Σ k (env&tgt~!y) (key!:e'$d)) di df b de
               where
                 (y,k)=e@!(i,be); be=ms n xϵ
                 e'=asS y
-wDOp (Lam (TyArr _ (TyB TyI)) n e) key src tgt (Σ i env d di df b) =
+wDOp (Lam (TyArr _ (TyB TyI)) n e) key src tgt (Σ i env d di df b de) =
     case env!src of
-        Nothing -> Σ i (tgt\~env) d di df b
+        Nothing -> Σ i (tgt\~env) d di df b de
         Just xϵ ->
             case IM.lookup key di of
-                Nothing -> Σ k (env&tgt~!y) d (IM.insert key (IS.singleton e') di) df b
-                Just ds -> (if e' `IS.member` ds then Σ k (tgt\~env) d di else Σ k (env&tgt~!y) d (IM.alter go key di)) df b
+                Nothing -> Σ k (env&tgt~!y) d (IM.insert key (IS.singleton e') di) df b de
+                Just ds -> (if e' `IS.member` ds then Σ k (tgt\~env) d di else Σ k (env&tgt~!y) d (IM.alter go key di)) df b de
 
               where
                 (y,k)=e@!(i,be); be=ms n xϵ
@@ -629,16 +607,25 @@ wDOp (Lam (TyArr _ (TyB TyI)) n e) key src tgt (Σ i env d di df b) =
 
                 go Nothing  = Just$!IS.singleton e'
                 go (Just s) = Just$!IS.insert e' s
-wDOp (Lam (TyArr _ (TyB TyFloat)) n e) key src tgt (Σ i env d di df b) =
+wDOp (Lam (TyArr _ (TyB TyFloat)) n e) key src tgt (Σ i env d di df b de) =
     case env!src of
-        Nothing -> Σ i (tgt\~env) d di df b
+        Nothing -> Σ i (tgt\~env) d di df b de
         Just xϵ ->
             case IM.lookup key df of
-                Nothing -> Σ k (env&tgt~!y) d di (IM.insert key (S.singleton e') df) b
-                Just ds -> if e' `S.member` ds then Σ k (tgt\~env) d di df b else Σ k (env&tgt~!y) d di (key!:e'$df) b
+                Nothing -> Σ k (env&tgt~!y) d di (IM.insert key (S.singleton e') df) b de
+                Just ds -> (if e' `S.member` ds then Σ k (tgt\~env) d di df else Σ k (env&tgt~!y) d di (key!:e'$df)) b de
               where
                 (y,k)=e@!(i,be); be=ms n xϵ
                 e'=asF y
+wDOp (Lam _ n e) key src tgt (Σ i env d di df b de) =
+    case env!src of
+        Nothing -> Σ i (tgt\~env) d di df b de
+        Just xϵ ->
+            case IM.lookup key de of
+                Nothing -> Σ k (env&tgt~!y) d di df b (IM.insert key (S.singleton y) de)
+                Just ds -> (if y `S.member` ds then Σ k (tgt\~env) d di df b de else Σ k (env&tgt~!y) d di df b (key!:y$de))
+              where
+                (y,k)=e@!(i,be); be=ms n xϵ
 wDOp e _ _ _ _ = throw $ InternalArityOrEta 1 e
 
 (\~) k = IM.insert k Nothing
@@ -647,60 +634,70 @@ wDOp e _ _ _ _ = throw $ InternalArityOrEta 1 e
 (!:) k e = IM.alter (\x -> Just$!case x of Nothing -> S.singleton e; Just s -> S.insert e s) k
 
 wB :: (E T, E T) -> Int -> Tmp -> Tmp -> Σ -> Σ
-wB (e0, e1) key src tgt (Σ i env d di df b) =
-    case env!src of
+wB (e0, e1) key src tgt (Σ i env d di df b de) =
+    (case env!src of
         Nothing -> Σ i (tgt\~env) d di df b
         Just xϵ -> let xS=asS xϵ in if key `IS.member` b
-            then if isMatch' r1 xS then Σ i (env&tgt~!xϵ) d di df (IS.delete key b) else Σ i (env&tgt~!xϵ) d di df b
-            else if isMatch' r0 xS then Σ i (env&tgt~!xϵ) d di df (IS.insert key b) else Σ i (tgt\~env) d di df b
+            then (if isMatch' r1 xS then Σ i (env&tgt~!xϵ) d di df (IS.delete key b) else Σ i (env&tgt~!xϵ) d di df b)
+            else (if isMatch' r0 xS then Σ i (env&tgt~!xϵ) d di df (IS.insert key b) else Σ i (tgt\~env) d di df b)) de
   where
     r0=asR e0; r1=asR e1
 
+wDE :: Int -> Tmp -> Tmp -> Σ -> Σ
+wDE key src tgt (Σ i env d di df b de) =
+    case env!src of
+        Nothing -> Σ i (tgt\~env) d di df b de
+        Just e ->
+            case IM.lookup key de of
+                Nothing -> Σ i (env&tgt~!e) d di df b (IM.insert key (S.singleton e) de)
+                Just ds -> if e `S.member` ds then Σ i (tgt\~env) d di df b de else Σ i (env&tgt~!e) d di df b (key!:e$de)
+
+-- TODO: TyB lol
 {-# SCC wD #-}
 wD :: TB -> Int -> Tmp -> Tmp -> Σ -> Σ
-wD TyStr key src tgt (Σ i env d di df b) =
+wD TyStr key src tgt (Σ i env d di df b de) =
     case env!src of
-        Nothing -> Σ i (tgt\~env) d di df b
+        Nothing -> Σ i (tgt\~env) d di df b de
         Just e ->
             case IM.lookup key d of
-                Nothing -> Σ i (env&tgt~!e) (IM.insert key (S.singleton e') d) di df b
-                Just ds -> (if e' `S.member` ds then Σ i (tgt\~env) d else Σ i (env&tgt~!e) (key!:e'$d)) di df b
+                Nothing -> Σ i (env&tgt~!e) (IM.insert key (S.singleton e') d) di df b de
+                Just ds -> (if e' `S.member` ds then Σ i (tgt\~env) d else Σ i (env&tgt~!e) (key!:e'$d)) di df b de
               where
                 e'=asS e
-wD TyI key src tgt (Σ i env d di df b) =
+wD TyI key src tgt (Σ i env d di df b de) =
     case env!src of
-        Nothing -> Σ i (tgt\~env) d di df b
+        Nothing -> Σ i (tgt\~env) d di df b de
         Just e ->
             case IM.lookup key di of
-                Nothing -> Σ i (env&tgt~!e) d (IM.insert key (IS.singleton e') di) df b
-                Just ds -> (if e' `IS.member` ds then Σ i (tgt\~env) d di else Σ i (env&tgt~!e) d (IM.alter go key di)) df b
+                Nothing -> Σ i (env&tgt~!e) d (IM.insert key (IS.singleton e') di) df b de
+                Just ds -> (if e' `IS.member` ds then Σ i (tgt\~env) d di else Σ i (env&tgt~!e) d (IM.alter go key di)) df b de
               where
                 e'=fromIntegral$asI e
 
                 go Nothing  = Just$!IS.singleton e'
                 go (Just s) = Just$!IS.insert e' s
-wD TyFloat key src tgt (Σ i env d di df b) =
+wD TyFloat key src tgt (Σ i env d di df b de) =
     case env!src of
-        Nothing -> Σ i (tgt\~env) d di df b
+        Nothing -> Σ i (tgt\~env) d di df b de
         Just e ->
             case IM.lookup key df of
-                Nothing -> Σ i (env&tgt~!e) d di (IM.insert key (S.singleton e') df) b
-                Just ds -> (if e' `S.member` ds then Σ i (tgt\~env) d di df else Σ i (env&tgt~!e) d di (key!:e'$df)) b
+                Nothing -> Σ i (env&tgt~!e) d di (IM.insert key (S.singleton e') df) b de
+                Just ds -> (if e' `S.member` ds then Σ i (tgt\~env) d di df else Σ i (env&tgt~!e) d di (key!:e'$df)) b de
               where
                 e'=asF e
 
 
 wP :: E T -> Tmp -> Tmp -> Σ -> Σ
-wP (Lam _ n e) src tgt (Σ j env d di df b) =
+wP (Lam _ n e) src tgt (Σ j env d di df b de) =
     case env!src of
         Just x ->
             let be=ms n x; (p,k)=e@!(j,be)
-            in Σ k (IM.insert tgt (if asB p then Just$!x else Nothing) env) d di df b
-        Nothing -> Σ j (tgt\~env) d di df b
+            in Σ k (IM.insert tgt (if asB p then Just$!x else Nothing) env) d di df b de
+        Nothing -> Σ j (tgt\~env) d di df b de
 wP e _ _ _ = throw $ InternalArityOrEta 1 e
 
 wΠ :: E T -> Tmp -> Tmp -> Tmp -> Σ -> Σ
-wΠ (Lam _ nn (Lam _ nprev e)) pt src tgt (Σ j env d di df b) =
+wΠ (Lam _ nn (Lam _ nprev e)) pt src tgt (Σ j env d di df b de) =
     (case (env!pt, env!src) of
         (Just prev, Just x) ->
             let be=me [(nprev, prev), (nn, x)]
@@ -708,12 +705,12 @@ wΠ (Lam _ nn (Lam _ nprev e)) pt src tgt (Σ j env d di df b) =
             in Σ u (IM.insert pt (Just$!x) (IM.insert tgt (Just$!res) env))
         (Nothing, Nothing) -> Σ j (tgt\~env)
         (Nothing, Just x) -> Σ j (pt~!x$tgt\~env)
-        (Just{}, Nothing) -> Σ j (tgt\~env)) d di df b
+        (Just{}, Nothing) -> Σ j (tgt\~env)) d di df b de
 wΠ e _ _ _ _ = throw $ InternalArityOrEta 2 e
 
 {-# SCC wF #-}
 wF :: E T -> Tmp -> Tmp -> Σ -> Σ
-wF (Lam _ nacc (Lam _ nn e)) src tgt (Σ j env d di df b) =
+wF (Lam _ nacc (Lam _ nn e)) src tgt (Σ j env d di df b de) =
     (case (env!tgt, env!src) of
         (Just acc, Just x) ->
             let be=me [(nacc, acc), (nn, x)]
@@ -721,7 +718,7 @@ wF (Lam _ nacc (Lam _ nn e)) src tgt (Σ j env d di df b) =
             in Σ u (env&tgt~!res)
         (Just acc, Nothing) -> Σ j (env&tgt~!acc)
         (Nothing, Nothing) -> Σ j (tgt\~env)
-        (Nothing, Just x) -> Σ j (env&tgt~!x)) d di df b
+        (Nothing, Just x) -> Σ j (env&tgt~!x)) d di df b de
 wF e _ _ _ = throw $ InternalArityOrEta 2 e
 
 badctx e = error ("Internal error: κ called on" ++ show e)
