@@ -17,10 +17,12 @@ module Jacinda.Regex ( lazySplit
 
 import           Control.Exception        (Exception, throwIO)
 import           Control.Monad            ((<=<))
+import           Data.Bifunctor           (bimap, first)
 import qualified Data.ByteString          as BS
 import qualified Data.ByteString.Internal as BS
 import qualified Data.ByteString.Lazy     as BSL
 import qualified Data.Vector              as V
+import qualified DL
 import           Foreign.C.Types          (CSize)
 import           Foreign.ForeignPtr       (plusForeignPtr)
 import           Regex.Rure               (RureFlags, RureMatch (..), RurePtr, captures, compile, find, findCaptures, isMatch, matches', rureDefaultFlags, rureFlagDotNL)
@@ -35,7 +37,7 @@ defaultRurePtr :: RurePtr
 defaultRurePtr = unsafePerformIO $ yIO =<< compile genFlags defaultFs
 
 genFlags :: RureFlags
-genFlags = rureDefaultFlags <> rureFlagDotNL -- in case they want to use a custom record separator
+genFlags = rureDefaultFlags <> rureFlagDotNL
 
 substr :: BS.ByteString -> Int -> Int -> BS.ByteString
 substr (BS.BS fp l) begin endϵ | endϵ >= begin = BS.BS (fp `plusForeignPtr` begin) (min l endϵ-begin)
@@ -44,9 +46,8 @@ substr (BS.BS fp l) begin endϵ | endϵ >= begin = BS.BS (fp `plusForeignPtr` be
 captures' :: RurePtr -> BS.ByteString -> CSize -> [BS.ByteString]
 captures' re haystack@(BS.BS fp _) ix = unsafeDupablePerformIO $ fmap go <$> captures re haystack ix
     where go (RureMatch s e) =
-            let e' = fromIntegral e
-                s' = fromIntegral s
-                in BS.BS (fp `plusForeignPtr` s') (e'-s')
+            let e' = fromIntegral e; s' = fromIntegral s
+            in BS.BS (fp `plusForeignPtr` s') (e'-s')
 
 {-# NOINLINE capturesIx #-}
 capturesIx :: RurePtr -> BS.ByteString -> CSize -> [RureMatch]
@@ -78,44 +79,40 @@ find' :: RurePtr -> BS.ByteString -> Maybe RureMatch
 find' re str = unsafeDupablePerformIO $ find re str 0
 
 lazySplitH :: RurePtr -> BSL.ByteString -> [BS.ByteString]
-lazySplitH rp = go Nothing . BSL.toChunks where
-    go Nothing [] = []
+lazySplitH rp = DL.toList . go Nothing . BSL.toChunks where
+    go Nothing [] = DL.empty
     go Nothing (c:cs) =
-        case unsnoc (splitH rp c) of
-            Just (iss,lss) -> iss++go (Just lss) cs
+        case splitH rp c of
+            Just (iss,lss) -> iss<>go (Just lss) cs
             Nothing        -> go Nothing cs
-    go (Just c) [] = splitByA rp c
+    go (Just c) [] = splitHLast rp c
     go (Just e) (c:cs) =
-        case unsnoc (splitByA rp (e<>c)) of
-            Just (iss,lss) -> iss++go (Just lss) cs
+        case splitH rp (e<>c) of
+            Just (iss,lss) -> iss<>go (Just lss) cs
             Nothing        -> go Nothing cs
 
 lazySplit :: RurePtr -> BSL.ByteString -> [BS.ByteString]
-lazySplit rp = go Nothing . BSL.toChunks where
-    go Nothing []      = []
+lazySplit rp = DL.toList . go Nothing . BSL.toChunks where
+    go Nothing []      = DL.empty
     go Nothing (c:cs)  =
-        case unsnoc (splitByA rp c) of
-            Just (iss,lss) -> iss++go (Just lss) cs
+        case splitByDL rp c of
+            Just (iss,lss) -> iss<>go (Just lss) cs
             Nothing        -> go Nothing cs
-    go (Just c) []     = splitByA rp c
+    go (Just c) []     = splitByL rp c
     go (Just e) (c:cs) =
-        case unsnoc (splitByA rp (e<>c)) of
-            Just (iss,lss) -> iss++go (Just lss) cs
+        case splitByDL rp (e<>c) of
+            Just (iss,lss) -> iss<>go (Just lss) cs
             Nothing        -> go Nothing cs
 
-unsnoc :: [a] -> Maybe ([a], a)
-unsnoc = foldr (\x acc -> Just $ case acc of {Nothing -> ([], x); Just ~(a, b) -> (x:a, b)}) Nothing
+{-# NOINLINE splitByL #-}
+splitByL :: RurePtr -> BS.ByteString -> DL.DL BS.ByteString
+splitByL rp b = case splitByDL rp b of {Nothing -> DL.empty; Just (as,a) -> as `DL.snoc` a}
 
+{-# NOINLINE splitBy #-}
 splitBy :: RurePtr -> BS.ByteString -> V.Vector BS.ByteString
-splitBy = (V.fromList .) . splitByA
-
-{-# NOINLINE splitByA #-}
-splitByA :: RurePtr
-         -> BS.ByteString
-         -> [BS.ByteString]
-splitByA _ "" = []
-splitByA re haystack@(BS.BS fp l) =
-    [BS.BS (fp `plusForeignPtr` s) (e-s) | (s,e) <- slicePairs]
+splitBy _ "" = []
+splitBy re haystack@(BS.BS fp l) =
+    V.fromList [ BS.BS (fp `plusForeignPtr` s) (e-s) | (s,e) <- slicePairs ]
     where ixes = unsafeDupablePerformIO $ matches' re haystack
           slicePairs = case ixes of
                 (RureMatch 0 i:rms) -> mkMiddle (fromIntegral i) rms
@@ -123,18 +120,34 @@ splitByA re haystack@(BS.BS fp l) =
           mkMiddle begin' []        = [(begin', l)]
           mkMiddle begin' (rm0:rms) = (begin', fromIntegral (start rm0)) : mkMiddle (fromIntegral $ end rm0) rms
 
-{-# NOINLINE splitH #-}
-splitH :: RurePtr -> BS.ByteString -> [BS.ByteString]
-splitH _ "" = []
-splitH re haystack@(BS.BS fp l) =
-    [BS.BS (fp `plusForeignPtr` s) (e-s) | (s,e) <- chopAt 0 ixes]
+{-# SCC splitByDL #-}
+{-# NOINLINE splitByDL #-}
+splitByDL :: RurePtr -> BS.ByteString
+          -> Maybe (DL.DL BS.ByteString, BS.ByteString)
+splitByDL _ "" = Nothing
+splitByDL re haystack@(BS.BS fp l) = bimap (fmap pp) pp <$> slicePairs
     where ixes = unsafeDupablePerformIO $ matches' re haystack
-          chopAt begin []                  = [(begin, l)]
-          chopAt begin (RureMatch b _:rms) = (begin, fromIntegral b) : chopAt (fromIntegral b) rms
+          slicePairs = case ixes of
+                (RureMatch 0 i:rms) -> mkMiddle (fromIntegral i) rms
+                rms                 -> mkMiddle 0 rms
+          mkMiddle begin' []        = Just (DL.empty, (begin', l))
+          mkMiddle begin' (rm0:rms) = first ((begin', fromIntegral (start rm0)) `DL.cons`) <$> mkMiddle (fromIntegral $ end rm0) rms
+          pp (s,e) = BS.BS (fp `plusForeignPtr` s) (e-s)
 
-isMatch' :: RurePtr
-         -> BS.ByteString
-         -> Bool
+{-# NOINLINE splitHLast #-}
+splitHLast :: RurePtr -> BS.ByteString -> DL.DL BS.ByteString
+splitHLast rp b = case splitH rp b of {Nothing -> DL.empty; Just (as,a) -> as `DL.snoc` a}
+
+{-# NOINLINE splitH #-}
+splitH :: RurePtr -> BS.ByteString -> Maybe (DL.DL BS.ByteString, BS.ByteString)
+splitH _ "" = Nothing
+splitH re haystack@(BS.BS fp l) = bimap (fmap pp) pp <$> chopAt 0 ixes
+    where ixes = unsafeDupablePerformIO $ matches' re haystack
+          chopAt begin []                  = Just (DL.empty, (begin, l))
+          chopAt begin (RureMatch b _:rms) = let b'=fromIntegral b in first ((begin, b') `DL.cons`) <$> chopAt b' rms
+          pp (s,e) = BS.BS (fp `plusForeignPtr` s) (e-s)
+
+isMatch' :: RurePtr -> BS.ByteString -> Bool
 isMatch' re haystack = unsafeDupablePerformIO $ isMatch re haystack 0
 
 compileDefault :: BS.ByteString -> RurePtr
@@ -143,7 +156,6 @@ compileDefault = unsafeDupablePerformIO . (yIO <=< compile genFlags)
 newtype RureExe = RegexCompile String
 
 instance Show RureExe where show (RegexCompile str) = str
-
 instance Exception RureExe where
 
 yIO :: Either String a -> IO a
