@@ -1,6 +1,4 @@
-module Nm ( Nm (..), TyName
-          , eqName
-          ) where
+module Nm ( Nm (..), TyName ) where
 
 import qualified Data.Text     as T
 import           Prettyprinter (Pretty (pretty))
@@ -11,13 +9,12 @@ data Nm a = Nm { name   :: T.Text
                , loc    :: a
                } deriving (Functor)
 
--- test
-eqName :: Nm a -> Nm a -> Bool
-eqName (Nm n _ _) (Nm n' _ _) = n == n'
-
 instance Eq (Nm a) where
     (==) (Nm _ u _) (Nm _ u' _) = u == u'
 
+instance Ord (Nm a) where
+    compare (Nm _ u _) (Nm _ u' _) = compare u u'
+
 instance Pretty (Nm a) where
     pretty (Nm t _ _) = pretty t
 
