@@ -4,15 +4,9 @@ module Ty.Const ( tyStream, tyOpt, tyV
 
 import           A
 
--- | argument assumed to have kind 'Star'
-tyStream :: T -> T
-tyStream = (TyB TyStream :$)
-
 tyB, tyI, tyF, tyStr, tyR :: T
 tyB=TyB TyBool; tyI=TyB TyI; tyF=TyB TyFloat; tyStr=TyB TyStr; tyR=TyB TyR
 
-tyOpt :: T -> T
-tyOpt = (TyB TyOption :$)
-
-tyV :: T -> T
-tyV = (TyB TyVec :$)
+-- | argument assumed to have kind 'Star'
+tyOpt, tyV, tyStream :: T -> T
+tyOpt = (TyB TyOption:$); tyStream = (TyB TyStream:$); tyV = (TyB TyVec:$)
